@@ -20,7 +20,7 @@ from typing import (
     Tuple,
     Type,
     Union,
-    cast,
+    cast, Iterable,
 )
 
 
@@ -67,6 +67,13 @@ class Will:
         self.properties = properties
 
 
+# See the overloads of `socket.setsockopt` for details.
+SocketOption = Union[
+    Tuple[int, int, Union[int, bytes]],
+    Tuple[int, int, None, int],
+]
+
+
 class Client:
     def __init__(
         self,
@@ -87,6 +94,8 @@ class Client:
         bind_port: int = 0,
         clean_start: bool = mqtt.MQTT_CLEAN_START_FIRST_ONLY,
         properties: Optional[Properties] = None,
+        message_retry_set: int = 20,
+        socket_options: Optional[Iterable[SocketOption]] = (),
     ):
         self._hostname = hostname
         self._port = port
@@ -113,6 +122,7 @@ class Client:
             protocol=protocol,
             clean_session=clean_session,
             transport=transport,
+            reconnect_on_failure=False,
         )
         self._client.on_connect = self._on_connect
         self._client.on_disconnect = self._on_disconnect
@@ -141,6 +151,9 @@ class Client:
                 will.topic, will.payload, will.qos, will.retain, will.properties
             )
 
+        self._client.message_retry_set(message_retry_set)
+        self._socket_options: Tuple[SocketOption] = tuple(socket_options)
+
     @property
     def id(self) -> str:
         """Return the client ID.
@@ -176,7 +189,7 @@ class Client:
                 self._clean_start, self._properties
             )
             client_socket = self._client.socket()
-            _set_client_socket_defaults(client_socket)
+            _set_client_socket_defaults(client_socket, self._socket_options)
         # paho.mqtt.Client.connect may raise one of several exceptions.
         # We convert all of them to the common MqttError for user convenience.
         # See: https://github.com/eclipse/paho.mqtt.python/blob/v1.5.0/src/paho/mqtt/client.py#L1770
@@ -193,7 +206,8 @@ class Client:
         await self._wait_for(self._disconnected, timeout=timeout)
 
     async def force_disconnect(self) -> None:
-        self._disconnected.set_result(None)
+        if not self._disconnected.done():
+            self._disconnected.set_result(None)
 
     async def subscribe(self, *args: Any, timeout: int = 10, **kwargs: Any) -> int:
         result, mid = self._client.subscribe(*args, **kwargs)
@@ -457,7 +471,7 @@ class Client:
     ) -> None:
         def cb() -> None:
             # client.loop_read() may raise an exception, such as BadPipe. It's
-            # usually a sign that the underlaying connection broke, therefore we 
+            # usually a sign that the underlaying connection broke, therefore we
             # disconnect straight away
             try:
                 client.loop_read()
@@ -489,7 +503,7 @@ class Client:
     ) -> None:
         def cb() -> None:
             # client.loop_write() may raise an exception, such as BadPipe. It's
-            # usually a sign that the underlaying connection broke, therefore we 
+            # usually a sign that the underlaying connection broke, therefore we
             # disconnect straight away
             try:
                 client.loop_write()
@@ -538,7 +552,8 @@ class Client:
 _PahoSocket = Union[socket.socket, mqtt.WebsocketWrapper]
 
 
-def _set_client_socket_defaults(client_socket: Optional[_PahoSocket]) -> None:
+def _set_client_socket_defaults(client_socket: Optional[_PahoSocket],
+                                socket_options: Iterable[SocketOption]) -> None:
     # Note that socket may be None if, e.g., the username and
     # password combination didn't work. In this case, we return early.
     if client_socket is None:
@@ -549,4 +564,5 @@ def _set_client_socket_defaults(client_socket: Optional[_PahoSocket]) -> None:
         return
     # At this point, we know that we got an actual socket. We change
     # some of the default options.
-    client_socket.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2048)
+    for socket_option in socket_options:
+        client_socket.setsockopt(*socket_option)
