@@ -1,20 +1,35 @@
 # SPDX-License-Identifier: BSD-3-Clause
 
 
+from typing import Any, Dict, Union
+
+import paho.mqtt.client as mqtt  # type: ignore
+
+
 class MqttError(Exception):
     """Base exception for all asyncio-mqtt exceptions."""
+
     pass
 
+
 class MqttCodeError(MqttError):
-    def __init__(self, rc, *args):
+    def __init__(self, rc: Union[int, mqtt.ReasonCodes], *args: Any):
         super().__init__(*args)
         self.rc = rc
-    
-    def __str__(self):
-        return f'[code:{self.rc}] {super().__str__()}'
+
+    def __str__(self) -> str:
+        if isinstance(self.rc, mqtt.ReasonCodes):
+            return f"[code:{self.rc.value}] {str(self.rc)}"
+        elif isinstance(self.rc, int):
+            return f"[code:{self.rc}] {mqtt.error_string(self.rc)}"
+        else:
+            return f"[code:{self.rc}] {super().__str__()}"
+
 
 class MqttConnectError(MqttCodeError):
-    def __init__(self, rc):
+    def __init__(self, rc: Union[int, mqtt.ReasonCodes]):
+        if isinstance(rc, mqtt.ReasonCodes):
+            return super().__init__(rc)
         msg = "Connection refused"
         try:
             msg += f": {_CONNECT_RC_STRINGS[rc]}"
@@ -22,7 +37,8 @@ class MqttConnectError(MqttCodeError):
             pass
         super().__init__(rc, msg)
 
-_CONNECT_RC_STRINGS = {
+
+_CONNECT_RC_STRINGS: Dict[int, str] = {
     # Reference: https://github.com/eclipse/paho.mqtt.python/blob/v1.5.0/src/paho/mqtt/client.py#L1898
     # 0: Connection successful
     # 1: Connection refused - incorrect protocol version
